@@ -1,5 +1,7 @@
 local api = vim.api
 
+local Range = require('vim.treesitter._range')
+
 local M = {}
 
 ---@class (private) vim.treesitter.dev.TSTreeView
@@ -76,10 +78,16 @@ end
 ---
 ---@package
 function TSTreeView:new(bufnr, lang)
-  local ok, parser = pcall(vim.treesitter.get_parser, bufnr or 0, lang)
-  if not ok then
-    local err = parser --[[ @as string ]]
-    return nil, 'No parser available for the given buffer:\n' .. err
+  bufnr = bufnr or 0
+  lang = lang or vim.treesitter.language.get_lang(vim.bo[bufnr].filetype)
+  local parser = vim.treesitter.get_parser(bufnr, lang, { error = false })
+  if not parser then
+    return nil,
+      string.format(
+        'Failed to create TSTreeView for buffer %s: no parser for lang "%s"',
+        bufnr,
+        lang
+      )
   end
 
   -- For each child tree (injected language), find the root of the tree and locate the node within
@@ -90,16 +98,20 @@ function TSTreeView:new(bufnr, lang)
 
   parser:for_each_tree(function(parent_tree, parent_ltree)
     local parent = parent_tree:root()
+    local parent_range = { parent:range() }
     for _, child in pairs(parent_ltree:children()) do
       for _, tree in pairs(child:trees()) do
         local r = tree:root()
-        local node = assert(parent:named_descendant_for_range(r:range()))
-        local id = node:id()
-        if not injections[id] or r:byte_length() > injections[id].root:byte_length() then
-          injections[id] = {
-            lang = child:lang(),
-            root = r,
-          }
+        local r_range = { r:range() }
+        if Range.contains(parent_range, r_range) then
+          local node = assert(parent:named_descendant_for_range(r:range()))
+          local id = node:id()
+          if not injections[id] or r:byte_length() > injections[id].root:byte_length() then
+            injections[id] = {
+              lang = child:lang(),
+              root = r,
+            }
+          end
         end
       end
     end
@@ -115,7 +127,7 @@ function TSTreeView:new(bufnr, lang)
   end
 
   local t = {
-    ns = api.nvim_create_namespace('treesitter/dev-inspect'),
+    ns = api.nvim_create_namespace('nvim.treesitter.dev_inspect'),
     nodes = nodes,
     named = named,
     ---@type vim.treesitter.dev.TSTreeViewOpts
@@ -131,15 +143,7 @@ function TSTreeView:new(bufnr, lang)
   return t
 end
 
-local decor_ns = api.nvim_create_namespace('ts.dev')
-
----@param range Range4
----@return string
-local function range_to_string(range)
-  ---@type integer, integer, integer, integer
-  local row, col, end_row, end_col = unpack(range)
-  return string.format('[%d, %d] - [%d, %d]', row, col, end_row, end_col)
-end
+local decor_ns = api.nvim_create_namespace('nvim.treesitter.dev')
 
 ---@param w integer
 ---@return boolean closed Whether the window was closed.
@@ -181,7 +185,7 @@ end
 --- @param source_buf integer
 --- @param inspect_buf integer
 --- @param inspect_win integer
---- @param pos? { [1]: integer, [2]: integer }
+--- @param pos? [integer, integer]
 local function set_inspector_cursor(treeview, lang, source_buf, inspect_buf, inspect_win, pos)
   api.nvim_buf_clear_namespace(inspect_buf, treeview.ns, 0, -1)
 
@@ -190,6 +194,7 @@ local function set_inspector_cursor(treeview, lang, source_buf, inspect_buf, ins
     lang = lang,
     pos = pos,
     ignore_injections = false,
+    include_anonymous = treeview.opts.anon,
   })
   if not cursor_node then
     return
@@ -222,14 +227,17 @@ function TSTreeView:draw(bufnr)
   local lang_hl_marks = {} ---@type table[]
 
   for i, item in self:iter() do
-    local range_str = range_to_string({ item.node:range() })
+    local range_str = ('[%d, %d] - [%d, %d]'):format(item.node:range())
     local lang_str = self.opts.lang and string.format(' %s', item.lang) or ''
 
     local text ---@type string
     if item.node:named() then
-      text = string.format('(%s', item.node:type())
+      text = string.format('(%s%s', item.node:missing() and 'MISSING ' or '', item.node:type())
     else
       text = string.format('%q', item.node:type()):gsub('\n', 'n')
+      if item.node:missing() then
+        text = string.format('(MISSING %s)', text)
+      end
     end
     if item.field then
       text = string.format('%s: %s', item.field, text)
@@ -325,27 +333,36 @@ end
 ---
 --- @param opts vim.treesitter.dev.inspect_tree.Opts?
 function M.inspect_tree(opts)
-  vim.validate({
-    opts = { opts, 't', true },
-  })
+  vim.validate('opts', opts, 'table', true)
 
   opts = opts or {}
 
+  -- source buffer
   local buf = api.nvim_get_current_buf()
+
+  -- window id for source buffer
   local win = api.nvim_get_current_win()
-  local treeview = assert(TSTreeView:new(buf, opts.lang))
+  local treeview, err = TSTreeView:new(buf, opts.lang)
+  if err and err:match('no parser for lang') then
+    vim.api.nvim_echo({ { err, 'WarningMsg' } }, true, {})
+    return
+  elseif not treeview then
+    error(err)
+  end
 
   -- Close any existing inspector window
   if vim.b[buf].dev_inspect then
     close_win(vim.b[buf].dev_inspect)
   end
 
+  -- window id for tree buffer
   local w = opts.winid
   if not w then
     vim.cmd(opts.command or '60vnew')
     w = api.nvim_get_current_win()
   end
 
+  -- tree buffer
   local b = opts.bufnr
   if b then
     api.nvim_win_set_buf(w, b)
@@ -381,6 +398,12 @@ function M.inspect_tree(opts)
     callback = function()
       local row = api.nvim_win_get_cursor(w)[1]
       local lnum, col = treeview:get(row).node:start()
+
+      -- update source window if original was closed
+      if not api.nvim_win_is_valid(win) then
+        win = vim.fn.win_findbuf(buf)[1]
+      end
+
       api.nvim_set_current_win(win)
       api.nvim_win_set_cursor(win, { lnum + 1, col })
     end,
@@ -428,7 +451,9 @@ function M.inspect_tree(opts)
     end,
   })
 
-  local group = api.nvim_create_augroup('treesitter/dev', {})
+  api.nvim_buf_set_keymap(b, 'n', 'q', '<C-w>c', { desc = 'Close language tree window' })
+
+  local group = api.nvim_create_augroup('nvim.treesitter.dev', {})
 
   api.nvim_create_autocmd('CursorMoved', {
     group = group,
@@ -438,6 +463,7 @@ function M.inspect_tree(opts)
         return true
       end
 
+      w = api.nvim_get_current_win()
       api.nvim_buf_clear_namespace(buf, treeview.ns, 0, -1)
       local row = api.nvim_win_get_cursor(w)[1]
       local lnum, col, end_lnum, end_col = treeview:get(row).node:range()
@@ -447,6 +473,11 @@ function M.inspect_tree(opts)
         hl_group = 'Visual',
       })
 
+      -- update source window if original was closed
+      if not api.nvim_win_is_valid(win) then
+        win = vim.fn.win_findbuf(buf)[1]
+      end
+
       local topline, botline = vim.fn.line('w0', win), vim.fn.line('w$', win)
 
       -- Move the cursor if highlighted range is completely out of view
@@ -507,17 +538,27 @@ function M.inspect_tree(opts)
     end,
   })
 
-  api.nvim_create_autocmd('BufHidden', {
+  api.nvim_create_autocmd({ 'BufHidden', 'BufUnload', 'QuitPre' }, {
     group = group,
     buffer = buf,
-    once = true,
     callback = function()
-      close_win(w)
+      -- don't close inpector window if source buffer
+      -- has more than one open window
+      if #vim.fn.win_findbuf(buf) > 1 then
+        return
+      end
+
+      -- close all tree windows
+      for _, window in pairs(vim.fn.win_findbuf(b)) do
+        close_win(window)
+      end
+
+      return true
     end,
   })
 end
 
-local edit_ns = api.nvim_create_namespace('treesitter/dev-edit')
+local edit_ns = api.nvim_create_namespace('nvim.treesitter.dev_edit')
 
 ---@param query_win integer
 ---@param base_win integer
@@ -525,7 +566,7 @@ local edit_ns = api.nvim_create_namespace('treesitter/dev-edit')
 local function update_editor_highlights(query_win, base_win, lang)
   local base_buf = api.nvim_win_get_buf(base_win)
   local query_buf = api.nvim_win_get_buf(query_win)
-  local parser = vim.treesitter.get_parser(base_buf, lang)
+  local parser = assert(vim.treesitter.get_parser(base_buf, lang, { error = false }))
   api.nvim_buf_clear_namespace(base_buf, edit_ns, 0, -1)
   local query_content = table.concat(api.nvim_buf_get_lines(query_buf, 0, -1, false), '\n')
 
@@ -542,10 +583,13 @@ local function update_editor_highlights(query_win, base_win, lang)
   -- Remove the '@' from the cursor word
   cursor_word = cursor_word:sub(2)
   local topline, botline = vim.fn.line('w0', base_win), vim.fn.line('w$', base_win)
-  for id, node in query:iter_captures(parser:trees()[1]:root(), base_buf, topline - 1, botline) do
+  for id, node, metadata in
+    query:iter_captures(parser:trees()[1]:root(), base_buf, topline - 1, botline)
+  do
     local capture_name = query.captures[id]
     if capture_name == cursor_word then
-      local lnum, col, end_lnum, end_col = node:range()
+      local lnum, col, end_lnum, end_col =
+        Range.unpack4(vim.treesitter.get_range(node, base_buf, metadata[id]))
       api.nvim_buf_set_extmark(base_buf, edit_ns, lnum, col, {
         end_row = end_lnum,
         end_col = end_col,
@@ -560,6 +604,8 @@ end
 
 --- @private
 --- @param lang? string language to open the query editor for.
+--- @return boolean? `true` on success, `nil` on failure
+--- @return string? error message, if applicable
 function M.edit_query(lang)
   local buf = api.nvim_get_current_buf()
   local win = api.nvim_get_current_win()
@@ -582,9 +628,10 @@ function M.edit_query(lang)
   end
   vim.cmd(cmd)
 
-  local ok, parser = pcall(vim.treesitter.get_parser, buf, lang)
-  if not ok then
-    return nil, 'No parser available for the given buffer'
+  local parser = vim.treesitter.get_parser(buf, lang, { error = false })
+  if not parser then
+    return nil,
+      string.format('Failed to show query editor for buffer %s: no parser for lang "%s"', buf, lang)
   end
   lang = parser:lang()
 
@@ -600,7 +647,7 @@ function M.edit_query(lang)
   -- can infer the language later.
   api.nvim_buf_set_name(query_buf, string.format('%s/query_editor.scm', lang))
 
-  local group = api.nvim_create_augroup('treesitter/dev-edit', {})
+  local group = api.nvim_create_augroup('nvim.treesitter.dev_edit', {})
   api.nvim_create_autocmd({ 'TextChanged', 'InsertLeave' }, {
     group = group,
     buffer = query_buf,
@@ -639,10 +686,10 @@ function M.edit_query(lang)
       api.nvim_buf_clear_namespace(query_buf, edit_ns, 0, -1)
     end,
   })
-  api.nvim_create_autocmd('BufHidden', {
+  api.nvim_create_autocmd({ 'BufHidden', 'BufUnload' }, {
     group = group,
     buffer = buf,
-    desc = 'Close the editor window when the source buffer is hidden',
+    desc = 'Close the editor window when the source buffer is hidden or unloaded',
     once = true,
     callback = function()
       close_win(query_win)
@@ -658,6 +705,8 @@ function M.edit_query(lang)
   })
   vim.cmd('normal! G')
   vim.cmd.startinsert()
+
+  return true
 end
 
 return M
