@@ -17,6 +17,7 @@
 #include "nvim/bufwrite.h"
 #include "nvim/change.h"
 #include "nvim/channel.h"
+#include "nvim/errors.h"
 #include "nvim/eval.h"
 #include "nvim/eval/typval.h"
 #include "nvim/eval/typval_defs.h"
@@ -29,10 +30,10 @@
 #include "nvim/fileio.h"
 #include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
-#include "nvim/highlight.h"
 #include "nvim/highlight_defs.h"
 #include "nvim/macros_defs.h"
 #include "nvim/mark.h"
+#include "nvim/memline_defs.h"
 #include "nvim/memory.h"
 #include "nvim/message.h"
 #include "nvim/move.h"
@@ -449,7 +450,7 @@ int buf_write_all(buf_T *buf, bool forceit)
                           1, buf->b_ml.ml_line_count, NULL,
                           false, forceit, true, false));
   if (curbuf != old_curbuf) {
-    msg_source(HL_ATTR(HLF_W));
+    msg_source(HLF_W);
     msg(_("Warning: Entered other buffer unexpectedly (check autocommands)"), 0);
   }
   return retval;
@@ -733,7 +734,7 @@ void ex_compiler(exarg_T *eap)
 
   if (eap->forceit) {
     // ":compiler! {name}" sets global options
-    do_cmdline_cmd("command -nargs=* CompilerSet set <args>");
+    do_cmdline_cmd("command -nargs=* -keepscript CompilerSet set <args>");
   } else {
     // ":compiler! {name}" sets local options.
     // To remain backwards compatible "current_compiler" is always
@@ -887,6 +888,17 @@ void ex_drop(exarg_T *eap)
       if (curbuf->b_ml.ml_flags & ML_EMPTY) {
         ex_rewind(eap);
       }
+
+      // execute [+cmd]
+      if (eap->do_ecmd_cmd) {
+        bool did_set_swapcommand = set_swapcommand(eap->do_ecmd_cmd, 0);
+        do_cmdline(eap->do_ecmd_cmd, NULL, NULL, DOCMD_VERBOSE);
+        if (did_set_swapcommand) {
+          set_vim_var_string(VV_SWAPCOMMAND, NULL, -1);
+        }
+      }
+
+      // no need to execute [++opts] - they only apply for newly loaded buffers.
       return;
     }
   }
