@@ -227,6 +227,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = false,
@@ -267,6 +268,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = false,
@@ -307,6 +309,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = false,
@@ -347,6 +350,7 @@ describe('nvim_create_user_command', function()
         line1 = 10,
         line2 = 10,
         mods = 'confirm unsilent botright horizontal',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = true,
@@ -387,6 +391,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 42,
         mods = '',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = false,
@@ -427,6 +432,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '*',
         smods = {
           browse = false,
           confirm = false,
@@ -479,6 +485,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '?',
         smods = {
           browse = false,
           confirm = false,
@@ -520,6 +527,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '?',
         smods = {
           browse = false,
           confirm = false,
@@ -572,6 +580,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '0',
         smods = {
           browse = false,
           confirm = false,
@@ -612,6 +621,7 @@ describe('nvim_create_user_command', function()
         line1 = 1,
         line2 = 1,
         mods = '',
+        nargs = '0',
         smods = {
           browse = false,
           confirm = false,
@@ -651,6 +661,11 @@ describe('nvim_create_user_command', function()
     api.nvim_set_current_buf(bufnr)
     command('Hello')
     assert_alive()
+    eq(
+      'Invalid buffer id: 1234',
+      pcall_err(api.nvim_buf_create_user_command, 1234, 'Hello', '', {})
+    )
+    assert_alive()
   end)
 
   it('can use a Lua complete function', function()
@@ -676,6 +691,19 @@ describe('nvim_create_user_command', function()
     eq('Test bbb', fn.getcmdline())
   end)
 
+  it('no crash when Lua complete function errors #33447', function()
+    exec_lua([[
+      vim.api.nvim_create_user_command('Test','', {
+          nargs = 1,
+          complete = function() error() end
+      })
+    ]])
+    feed(':Test <Tab>')
+    eq('E5108: Error executing Lua function: [NULL]', api.nvim_get_vvar('errmsg'))
+    eq('Test ', fn.getcmdline())
+    assert_alive()
+  end)
+
   it('does not allow invalid command names', function()
     eq(
       "Invalid command name (must start with uppercase): 'test'",
@@ -754,6 +782,13 @@ describe('nvim_create_user_command', function()
     eq(5, #api.nvim_list_tabpages())
     eq(1, fn.tabpagenr())
   end)
+
+  it('"addr" flag allows ranges without explicit "range" flag', function()
+    exec_lua([[
+      vim.api.nvim_create_user_command('Test2', 'echo "hello <line1><line2>"', { addr = 'other' })
+    ]])
+    eq('hello 12', n.exec_capture('1,2Test2'))
+  end)
 end)
 
 describe('nvim_del_user_command', function()
@@ -771,5 +806,9 @@ describe('nvim_del_user_command', function()
     command('Hello')
     api.nvim_buf_del_user_command(0, 'Hello')
     matches('Not an editor command: Hello', pcall_err(command, 'Hello'))
+    eq('Invalid command (not found): Hello', pcall_err(api.nvim_buf_del_user_command, 0, 'Hello'))
+    eq('Invalid command (not found): Bye', pcall_err(api.nvim_buf_del_user_command, 0, 'Bye'))
+    eq('Invalid buffer id: 1234', pcall_err(api.nvim_buf_del_user_command, 1234, 'Hello'))
+    assert_alive()
   end)
 end)
