@@ -19,8 +19,8 @@ describe('cmdline autocommands', function()
     api.nvim_set_var('channel', channel)
     command("autocmd CmdlineEnter * call rpcnotify(g:channel, 'CmdlineEnter', v:event)")
     command("autocmd CmdlineLeave * call rpcnotify(g:channel, 'CmdlineLeave', v:event)")
-    command("autocmd CmdWinEnter * call rpcnotify(g:channel, 'CmdWinEnter', v:event)")
-    command("autocmd CmdWinLeave * call rpcnotify(g:channel, 'CmdWinLeave', v:event)")
+    command("autocmd CmdwinEnter * call rpcnotify(g:channel, 'CmdwinEnter', v:event)")
+    command("autocmd CmdwinLeave * call rpcnotify(g:channel, 'CmdwinLeave', v:event)")
   end)
 
   it('works', function()
@@ -60,13 +60,6 @@ describe('cmdline autocommands', function()
   it('handles errors correctly', function()
     clear()
     local screen = Screen.new(72, 8)
-    screen:attach()
-    screen:set_default_attr_ids({
-      [1] = { bold = true, foreground = Screen.colors.Blue1 },
-      [2] = { foreground = Screen.colors.Grey100, background = Screen.colors.Red },
-      [3] = { bold = true, foreground = Screen.colors.SeaGreen4 },
-      [4] = { bold = true, reverse = true },
-    })
     command("autocmd CmdlineEnter * echoerr 'FAIL'")
     command("autocmd CmdlineLeave * echoerr 'very error'")
 
@@ -74,22 +67,22 @@ describe('cmdline autocommands', function()
     screen:expect([[
                                                                               |
       {1:~                                                                       }|*3
-      {4:                                                                        }|
+      {3:                                                                        }|
       :                                                                       |
-      {2:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
+      {9:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
       :^                                                                       |
     ]])
 
     feed("put ='lorem ipsum'<cr>")
     screen:expect([[
                                                                               |
-      {4:                                                                        }|
+      {3:                                                                        }|
       :                                                                       |
-      {2:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
+      {9:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
       :put ='lorem ipsum'                                                     |
-      {2:CmdlineLeave Autocommands for "*": Vim(echoerr):very error}              |
+      {9:CmdlineLeave Autocommands for "*": Vim(echoerr):very error}              |
                                                                               |
-      {3:Press ENTER or type command to continue}^                                 |
+      {6:Press ENTER or type command to continue}^                                 |
     ]])
 
     -- cmdline was still executed
@@ -108,11 +101,11 @@ describe('cmdline autocommands', function()
     screen:expect([[
                                                                               |
       lorem ipsum                                                             |
-      {4:                                                                        }|
+      {3:                                                                        }|
       :                                                                       |
-      {2:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
+      {9:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
       :put ='lorem ipsum'                                                     |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum'^                                                     |
     ]])
 
@@ -120,37 +113,37 @@ describe('cmdline autocommands', function()
     screen:expect([[
                                                                               |
       lorem ipsum                                                             |
-      {4:                                                                        }|
+      {3:                                                                        }|
       :                                                                       |
-      {2:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
+      {9:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
       :put ='lorem ipsum'                                                     |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum^'                                                     |
     ]])
 
     -- edit still works
     feed('.')
     screen:expect([[
-      {4:                                                                        }|
+      {3:                                                                        }|
       :                                                                       |
-      {2:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
+      {9:CmdlineEnter Autocommands for "*": Vim(echoerr):FAIL}                    |
       :put ='lorem ipsum'                                                     |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum.'                                                    |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum.^'                                                    |
     ]])
 
     feed('<cr>')
     screen:expect([[
       :put ='lorem ipsum'                                                     |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum.'                                                    |
-      {2:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
+      {9:CmdlineChanged Autocommands for "*": Vim(echoerr):change erreor}         |
       :put ='lorem ipsum.'                                                    |
-      {2:CmdlineLeave Autocommands for "*": Vim(echoerr):very error}              |
+      {9:CmdlineLeave Autocommands for "*": Vim(echoerr):very error}              |
                                                                               |
-      {3:Press ENTER or type command to continue}^                                 |
+      {6:Press ENTER or type command to continue}^                                 |
     ]])
 
     -- cmdline was still executed
@@ -170,7 +163,7 @@ describe('cmdline autocommands', function()
     feed('<c-r>=')
     eq({ 'notification', 'CmdlineEnter', { { cmdtype = '=', cmdlevel = 2 } } }, next_msg())
     feed('<c-f>')
-    eq({ 'notification', 'CmdWinEnter', { {} } }, next_msg())
+    eq({ 'notification', 'CmdwinEnter', { {} } }, next_msg())
     feed(':')
     eq({ 'notification', 'CmdlineEnter', { { cmdtype = ':', cmdlevel = 3 } } }, next_msg())
     feed('<c-c>')
@@ -179,7 +172,7 @@ describe('cmdline autocommands', function()
       next_msg()
     )
     feed('<c-c>')
-    eq({ 'notification', 'CmdWinLeave', { {} } }, next_msg())
+    eq({ 'notification', 'CmdwinLeave', { {} } }, next_msg())
     feed('1+2<cr>')
     eq(
       { 'notification', 'CmdlineLeave', { { cmdtype = '=', cmdlevel = 2, abort = false } } },
