@@ -1,10 +1,11 @@
-local helpers = require('test.functional.helpers')(after_each)
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
 
-local eq, neq, call = helpers.eq, helpers.neq, helpers.call
-local eval, feed, clear = helpers.eval, helpers.feed, helpers.clear
-local command, insert, expect = helpers.command, helpers.insert, helpers.expect
-local feed_command = helpers.feed_command
-local curwin = helpers.curwin
+local eq, neq, call = t.eq, t.neq, n.call
+local eval, feed, clear = n.eval, n.feed, n.clear
+local command, insert, expect = n.command, n.insert, n.expect
+local feed_command = n.feed_command
+local curwin = n.api.nvim_get_current_win
 
 describe("'langmap'", function()
   before_each(function()
@@ -14,13 +15,13 @@ describe("'langmap'", function()
     feed('gg0')
   end)
 
-  it("converts keys in normal mode", function()
+  it('converts keys in normal mode', function()
     feed('ix')
     expect('iii ww')
     feed('whello<esc>')
     expect('iii helloww')
   end)
-  it("gives characters that are mapped by :nmap.", function()
+  it('gives characters that are mapped by :nmap.', function()
     command('map i 0x')
     feed('w')
     expect('ii www')
@@ -32,20 +33,18 @@ describe("'langmap'", function()
     it("'langnoremap' is by default ON", function()
       eq(1, eval('&langnoremap'))
     end)
-    it("Results of maps are not converted when 'langnoremap' ON.",
-    function()
+    it("Results of maps are not converted when 'langnoremap' ON.", function()
       command('nmap x i')
       feed('xdl<esc>')
       expect('dliii www')
     end)
-    it("applies when deciding whether to map recursively", function()
+    it('applies when deciding whether to map recursively', function()
       command('nmap l i')
       command('nmap w j')
       feed('ll')
       expect('liii www')
     end)
-    it("does not stop applying 'langmap' on first character of a mapping",
-    function()
+    it("does not stop applying 'langmap' on first character of a mapping", function()
       command('1t1')
       command('1t1')
       command('goto 1')
@@ -56,8 +55,7 @@ describe("'langmap'", function()
       iii www
       ihelloii www]])
     end)
-    it("Results of maps are converted when 'langnoremap' OFF.",
-    function()
+    it("Results of maps are converted when 'langnoremap' OFF.", function()
       command('set nolangnoremap')
       command('nmap x i')
       feed('xdl<esc>')
@@ -65,8 +63,7 @@ describe("'langmap'", function()
     end)
   end)
   -- e.g. CTRL-W_j  ,  mj , 'j and "jp
-  it('conversions are applied to keys in middle of command',
-  function()
+  it('conversions are applied to keys in middle of command', function()
     -- Works in middle of window command
     feed('<C-w>s')
     local origwin = curwin()
@@ -74,12 +71,12 @@ describe("'langmap'", function()
     neq(origwin, curwin())
     -- Works when setting a mark
     feed('yy3p3gg0mwgg0mi')
-    eq({0, 3, 1, 0}, call('getpos', "'i"))
-    eq({0, 1, 1, 0}, call('getpos', "'w"))
+    eq({ 0, 3, 1, 0 }, call('getpos', "'i"))
+    eq({ 0, 1, 1, 0 }, call('getpos', "'w"))
     feed('3dd')
     -- Works when moving to a mark
     feed("'i")
-    eq({0, 1, 1, 0}, call('getpos', '.'))
+    eq({ 0, 1, 1, 0 }, call('getpos', '.'))
     -- Works when selecting a register
     feed('qillqqwhhq')
     eq('hh', eval('@i'))
@@ -137,7 +134,7 @@ describe("'langmap'", function()
       hello]])
     end)
     it('command-line CTRL-R', function()
-      helpers.source([[
+      n.source([[
         let i_value = 0
         let j_value = 0
         call setreg('i', 'i_value')
@@ -175,7 +172,7 @@ describe("'langmap'", function()
     end)
     it('prompt for number', function()
       command('set langmap=12,21')
-      helpers.source([[
+      n.source([[
         let gotten_one = 0
         function Map()
           let answer = inputlist(['a', '1.', '2.', '3.'])
@@ -193,8 +190,7 @@ describe("'langmap'", function()
       eq(1, eval('gotten_one'))
     end)
   end)
-  it('conversions are not applied during setreg()',
-  function()
+  it('conversions are not applied during setreg()', function()
     call('setreg', 'i', 'ww')
     eq('ww', eval('@i'))
   end)
@@ -214,12 +210,15 @@ describe("'langmap'", function()
   end)
 
   local function testrecording(command_string, expect_string, setup_function, expect_macro)
-    if setup_function then setup_function() end
+    if setup_function then
+      setup_function()
+    end
     feed('qa' .. command_string .. 'q')
     expect(expect_string)
-    eq(expect_macro or helpers.funcs.nvim_replace_termcodes(command_string, true, true, true),
-      eval('@a'))
-    if setup_function then setup_function() end
+    eq(expect_macro or n.fn.nvim_replace_termcodes(command_string, true, true, true), eval('@a'))
+    if setup_function then
+      setup_function()
+    end
     -- n.b. may need nvim_replace_termcodes() here.
     feed('@a')
     expect(expect_string)
@@ -267,6 +266,18 @@ describe("'langmap'", function()
     command('nnoremap ï ix<esc>')
     testrecording('x', 'xhello', local_setup)
   end)
+  it('does not crash when mapping char < 256 to char >= 256', function()
+    -- Note: this warning should be removed when the pending tests above are fixed.
+    local wmsg = table.concat({
+      "'langmap': Mapping from e to ε will not work properly",
+      "'langmap': Mapping from ü to μ will not work properly",
+    }, '\n')
+    -- ä ë ï ö ü ÿ < 256, β γ ε μ >= 256
+    command('set langmap=iw,wi,äë,ëä,ïx,xï,βγ,γβ,eε,εe,üμ,μü,ÿy,yÿ')
+    eq(wmsg, n.exec_capture('messages'))
+    command('messages clear | set langmap=iwäëïxβγeεüμÿy;wiëäxïγβεeμüyÿ')
+    eq(wmsg, n.exec_capture('messages'))
+  end)
   -- This test is to ensure the behaviour doesn't change from what's already
   -- around. I (hardenedapple) personally think this behaviour should be
   -- changed.
@@ -276,5 +287,4 @@ describe("'langmap'", function()
     testrecording('<C-w>', 'whello', local_setup, eval([["\<*C-w>"]]))
     testrecording('<C-i>', 'ihello', local_setup, eval([["\<*C-i>"]]))
   end)
-
 end)
