@@ -105,7 +105,6 @@ describe('tabpage', function()
     screen:add_extra_attr_ids {
       [100] = { bold = true, foreground = Screen.colors.Fuchsia },
     }
-    screen:attach()
 
     command('tabnew')
     command('tabprev')
@@ -138,9 +137,23 @@ describe('tabpage', function()
     eq(1, fn.nvim_tabpage_get_number(0))
   end)
 
+  it('0 means current tabpage for gettabvar()', function()
+    command('let t:tabvar = 42')
+    eq(42, fn.gettabvar(0, 'tabvar'))
+  end)
+
   it(':tabs does not overflow IObuff with long path with comma #20850', function()
     api.nvim_buf_set_name(0, ('x'):rep(1024) .. ',' .. ('x'):rep(1024))
     command('tabs')
     assert_alive()
   end)
+
+  it('no crash if autocmd remains in tabpage of closing last window', function()
+    exec([[
+      tabnew
+      let s:win = win_getid()
+      autocmd TabLeave * ++once tablast | tabonly
+      quit
+    ]])
+  end)
 end)
