@@ -1,17 +1,19 @@
+local t = require('test.testutil')
 local n = require('test.functional.testnvim')()
 local Screen = require('test.functional.ui.screen')
 
 local clear = n.clear
+local eq = t.eq
 local exec = n.exec
 local exec_lua = n.exec_lua
 local command = n.command
 local feed = n.feed
+local fn = n.fn
 
 -- oldtest: Test_window_cmd_ls0_split_scrolling()
 it('scrolling with laststatus=0 and :botright split', function()
   clear('--cmd', 'set ruler')
   local screen = Screen.new(40, 10)
-  screen:attach()
   exec([[
     set laststatus=0
     call setline(1, range(1, 100))
@@ -38,7 +40,6 @@ describe('splitkeep', function()
   before_each(function()
     clear('--cmd', 'set splitkeep=screen')
     screen = Screen.new()
-    screen:attach()
   end)
 
   -- oldtest: Test_splitkeep_cursor()
@@ -122,7 +123,7 @@ describe('splitkeep', function()
             row = 0,
             col = 0,
           }))
-          vim.cmd("call termopen([&sh, &shcf, 'true'], { 'on_exit': 'C2' })")
+          vim.cmd("call jobstart([&sh, &shcf, 'true'], { 'term': v:true, 'on_exit': 'C2' })")
       end
     })]])
     feed('j')
@@ -301,7 +302,7 @@ describe('splitkeep', function()
       c                                                    |
       {1:~                                                    }|
       {3:[No Name]                                            }|
-                                                           |
+      :call win_move_statusline(win, 1)                    |
     ]])
   end)
 
@@ -328,4 +329,105 @@ describe('splitkeep', function()
                                               |
     ]])
   end)
+
+  -- oldtest: Test_splitkeep_line()
+  it("no scrolling with 'splitkeep' and line()", function()
+    screen:try_resize(40, 6)
+    exec([[
+      set splitkeep=screen nosplitbelow
+      autocmd WinResized * call line('w0', 1000)
+      call setline(1, range(1000))
+    ]])
+    screen:expect([[
+      ^0                                       |
+      1                                       |
+      2                                       |
+      3                                       |
+      4                                       |
+                                              |
+    ]])
+    feed(':wincmd s<CR>')
+    screen:expect([[
+      ^0                                       |
+      1                                       |
+      {3:[No Name] [+]                           }|
+      3                                       |
+      {2:[No Name] [+]                           }|
+      :wincmd s                               |
+    ]])
+  end)
+end)
+
+-- oldtest: Test_winfixsize_vsep_statusline()
+it("'winfixwidth/height' does not leave stray vseps/statuslines", function()
+  clear()
+  local screen = Screen.new(75, 8)
+  exec([[
+    set noequalalways splitbelow splitright
+    vsplit
+    setlocal winfixwidth
+    vsplit
+  ]])
+  eq(16, fn.winwidth(1))
+  eq(37, fn.winwidth(2))
+  eq(20, fn.winwidth(3))
+
+  command('quit')
+  screen:expect([[
+    ^                                    │                                      |
+    {1:~                                   }│{1:~                                     }|*5
+    {3:[No Name]                            }{2:[No Name]                             }|
+                                                                               |
+  ]])
+  -- Checks w_view_width in Nvim; especially important as the screen test may still pass if only
+  -- w_width changed to make room for the vsep.
+  eq(36, fn.winwidth(1))
+  eq(38, fn.winwidth(2))
+
+  exec([[
+    set laststatus=0
+    only
+    split
+    set winfixheight
+    split
+  ]])
+  eq(1, fn.winheight(1))
+  eq(3, fn.winheight(2))
+  eq(1, fn.winheight(3))
+
+  command('quit')
+  screen:expect([[
+    ^                                                                           |
+    {1:~                                                                          }|
+    {3:[No Name]                                                                  }|
+                                                                               |
+    {1:~                                                                          }|*3
+                                                                               |
+  ]])
+  eq(2, fn.winheight(1))
+  eq(4, fn.winheight(2))
+end)
+
+-- oldtest: Test_resize_from_another_tabpage()
+it('resizing window from another tabpage', function()
+  clear()
+  local screen = Screen.new(75, 8)
+  screen:add_extra_attr_ids({
+    [100] = { foreground = Screen.colors.Magenta1, bold = true },
+  })
+  exec([[
+    set laststatus=2
+    vnew
+    let w = win_getid()
+    tabnew
+    call win_execute(w, 'vertical resize 20')
+    tabprev
+  ]])
+  screen:expect([[
+    {5: }{100:2}{5: [No Name] }{24: [No Name] }{2:                                                  }{24:X}|
+    ^                    │                                                      |
+    {1:~                   }│{1:~                                                     }|*4
+    {3:[No Name]            }{2:[No Name]                                             }|
+                                                                               |
+  ]])
 end)
