@@ -36,13 +36,17 @@ describe(':checkhealth', function()
     clear()
     -- Do this after startup, otherwise it just breaks $VIMRUNTIME.
     command("let $VIM='zub'")
-    command('checkhealth nvim')
+    command('checkhealth vim.health')
     matches('ERROR $VIM .* zub', curbuf_contents())
   end)
 
   it('getcompletion()', function()
     clear { args = { '-u', 'NORC', '+set runtimepath+=test/functional/fixtures' } }
 
+    eq('vim.deprecated', getcompletion('vim', 'checkhealth')[1])
+    eq('vim.provider', getcompletion('vim.prov', 'checkhealth')[1])
+    eq('vim.lsp', getcompletion('vim.ls', 'checkhealth')[1])
+
     -- "test_plug/health/init.lua" should complete as "test_plug", not "test_plug.health". #30342
     eq({
       'test_plug',
@@ -62,6 +66,46 @@ describe(':checkhealth', function()
     eq({}, getcompletion('', 'checkhealth'))
     assert_alive()
   end)
+
+  it('cmdline completion works with multiple args #35054', function()
+    clear()
+    n.feed(':checkhealth vim.ls<Tab>')
+    eq('checkhealth vim.lsp', fn.getcmdline())
+    n.feed(' vim.prov<Tab>')
+    eq('checkhealth vim.lsp vim.provider', fn.getcmdline())
+  end)
+
+  it('vim.g.health', function()
+    clear {
+      args_rm = { '-u' },
+      args = { '--clean', '+set runtimepath+=test/functional/fixtures' },
+    }
+    command("let g:health = {'style':'float'}")
+    command('checkhealth lsp')
+    eq(
+      'editor',
+      exec_lua([[
+      return vim.api.nvim_win_get_config(0).relative
+    ]])
+    )
+
+    -- gO should not close the :checkhealth floating window. #34784
+    command('checkhealth full_render')
+    local win = api.nvim_get_current_win()
+    api.nvim_win_set_cursor(win, { 5, 1 })
+    n.feed('gO')
+    eq(true, api.nvim_win_is_valid(win))
+    eq('qf', api.nvim_get_option_value('filetype', { buf = 0 }))
+  end)
+
+  it("vim.provider works with a misconfigured 'shell'", function()
+    clear()
+    command([[set shell=echo\ WRONG!!!]])
+    command('let g:loaded_perl_provider = 0')
+    command('let g:loaded_python3_provider = 0')
+    command('checkhealth vim.provider')
+    eq(nil, string.match(curbuf_contents(), 'WRONG!!!'))
+  end)
 end)
 
 describe('vim.health', function()
@@ -70,12 +114,39 @@ describe('vim.health', function()
   end)
 
   describe(':checkhealth', function()
-    it('functions report_*() render correctly', function()
+    it('report_xx() renders correctly', function()
       command('checkhealth full_render')
       n.expect([[
 
       ==============================================================================
-      test_plug.full_render: require("test_plug.full_render.health").check()
+      test_plug.full_render:                                              1 ⚠️  1 ❌
+
+      report 1 ~
+      - ✅ OK life is fine
+      - ⚠️ WARNING no what installed
+        - ADVICE:
+          - pip what
+          - make what
+
+      report 2 ~
+      - stuff is stable
+      - ❌ ERROR why no hardcopy
+        - ADVICE:
+          - :help |:hardcopy|
+          - :help |:TOhtml|
+      ]])
+    end)
+
+    it('user FileType handler can modify report', function()
+      -- Define a FileType autocmd that removes emoji chars.
+      source [[
+        autocmd FileType checkhealth :set modifiable | silent! %s/\v( ?[^\x00-\x7F])//g
+        checkhealth full_render
+      ]]
+      n.expect([[
+
+      ==============================================================================
+      test_plug.full_render:                                              1  1
 
       report 1 ~
       - OK life is fine
@@ -98,28 +169,28 @@ describe('vim.health', function()
       n.expect([[
 
         ==============================================================================
-        test_plug: require("test_plug.health").check()
+        test_plug:                                                                  ✅
 
         report 1 ~
-        - OK everything is fine
+        - ✅ OK everything is fine
 
         report 2 ~
-        - OK nothing to see here
+        - ✅ OK nothing to see here
 
         ==============================================================================
-        test_plug.success1: require("test_plug.success1.health").check()
+        test_plug.success1:                                                         ✅
 
         report 1 ~
-        - OK everything is fine
+        - ✅ OK everything is fine
 
         report 2 ~
-        - OK nothing to see here
+        - ✅ OK nothing to see here
 
         ==============================================================================
-        test_plug.success2: require("test_plug.success2.health").check()
+        test_plug.success2:                                                         ✅
 
         another 1 ~
-        - OK ok
+        - ✅ OK ok
         ]])
     end)
 
@@ -128,13 +199,13 @@ describe('vim.health', function()
       n.expect([[
 
         ==============================================================================
-        test_plug.submodule: require("test_plug.submodule.health").check()
+        test_plug.submodule:                                                        ✅
 
         report 1 ~
-        - OK everything is fine
+        - ✅ OK everything is fine
 
         report 2 ~
-        - OK nothing to see here
+        - ✅ OK nothing to see here
         ]])
     end)
 
@@ -143,19 +214,19 @@ describe('vim.health', function()
       n.expect([[
 
       ==============================================================================
-      test_plug.submodule_empty: require("test_plug.submodule_empty.health").check()
+      test_plug.submodule_empty:                                                1 ❌
 
-      - ERROR The healthcheck report for "test_plug.submodule_empty" plugin is empty.
+      - ❌ ERROR The healthcheck report for "test_plug.submodule_empty" plugin is empty.
       ]])
     end)
 
     it('highlights OK, ERROR', function()
       local screen = Screen.new(50, 12)
-      screen:attach()
       screen:set_default_attr_ids({
+        h1 = { reverse = true },
+        h2 = { foreground = tonumber('0x6a0dad') },
         Ok = { foreground = Screen.colors.LightGreen },
         Error = { foreground = Screen.colors.Red },
-        Heading = { foreground = tonumber('0x6a0dad') },
         Bar = { foreground = Screen.colors.LightGrey, background = Screen.colors.DarkGrey },
       })
       command('checkhealth foo success1')
@@ -163,16 +234,16 @@ describe('vim.health', function()
       screen:expect {
         grid = [[
         ^                                                  |
-        {Bar:──────────────────────────────────────────────────}|
-        {Heading:foo: }                                             |
+        {Bar:                                                  }|
+        {h1:foo:                                              }|
                                                           |
-        - {Error:ERROR} No healthcheck found for "foo" plugin.    |
+        - ❌ {Error:ERROR} No healthcheck found for "foo" plugin. |
                                                           |
-        {Bar:──────────────────────────────────────────────────}|
-        {Heading:test_plug.success1: require("test_plug.success1.he}|
+        {Bar:                                                  }|
+        {h1:test_plug.success1:                               }|
                                                           |
-        {Heading:report 1}                                          |
-        - {Ok:OK} everything is fine                           |
+        {h2:report 1}                                          |
+        - ✅ {Ok:OK} everything is fine                        |
                                                           |
       ]],
       }
@@ -184,29 +255,48 @@ describe('vim.health', function()
       n.expect([[
 
         ==============================================================================
-        non_existent_healthcheck: 
+        non_existent_healthcheck:                                                 1 ❌
 
-        - ERROR No healthcheck found for "non_existent_healthcheck" plugin.
+        - ❌ ERROR No healthcheck found for "non_existent_healthcheck" plugin.
         ]])
     end)
 
-    it('does not use vim.health as a healtcheck', function()
+    it('does not use vim.health as a healthcheck', function()
       -- vim.health is not a healthcheck
       command('checkhealth vim')
       n.expect([[
       ERROR: No healthchecks found.]])
     end)
-  end)
-end)
 
-describe(':checkhealth provider', function()
-  it("works correctly with a wrongly configured 'shell'", function()
-    clear()
-    command([[set shell=echo\ WRONG!!!]])
-    command('let g:loaded_perl_provider = 0')
-    command('let g:loaded_python3_provider = 0')
-    command('checkhealth provider')
-    eq(nil, string.match(curbuf_contents(), 'WRONG!!!'))
+    it('nested lua/ directory', function()
+      command('checkhealth lua')
+      n.expect([[
+
+      ==============================================================================
+      test_plug.lua:                                                              ✅
+
+      nested lua/ directory ~
+      - ✅ OK everything is ok
+      ]])
+    end)
+
+    it('&rtp can contain nested path (by packadd)', function()
+      -- re-add to ensure this appears before new nested rtp
+      command([[set runtimepath-=test/functional/fixtures]])
+      command([[set runtimepath+=test/functional/fixtures]])
+      command('set packpath+=test/functional/fixtures')
+      -- set rtp+=test/functional/fixtures/pack/foo/opt/healthy
+      command('packadd healthy')
+      command('checkhealth nest')
+      n.expect([[
+
+      ==============================================================================
+      nest:                                                                       ✅
+
+      healthy pack ~
+      - ✅ OK healthy ok
+      ]])
+    end)
   end)
 end)
 
@@ -217,14 +307,14 @@ describe(':checkhealth window', function()
   end)
 
   it('opens directly if no buffer created', function()
-    local screen = Screen.new(50, 12)
+    local screen = Screen.new(50, 12, { ext_multigrid = true })
     screen:set_default_attr_ids {
+      h1 = { reverse = true },
+      h2 = { foreground = tonumber('0x6a0dad') },
       [1] = { foreground = Screen.colors.Blue, bold = true },
       [14] = { foreground = Screen.colors.LightGrey, background = Screen.colors.DarkGray },
-      [31] = { foreground = tonumber('0x6a0dad') },
       [32] = { foreground = Screen.colors.PaleGreen2 },
     }
-    screen:attach({ ext_multigrid = true })
     command('checkhealth success1')
     screen:expect {
       grid = [[
@@ -233,16 +323,16 @@ describe(':checkhealth window', function()
       [3:--------------------------------------------------]|
     ## grid 2
       ^                                                  |
-      {14:──────────────────────────────────────────────────}|
-      {14:────────────────────────────}                      |
-      {31:test_plug.success1: require("test_plug.success1.  }|
-      {31:health").check()}                                  |
+      {14:                                                  }|
+      {14:                            }                      |
+      {h1:test_plug.                                        }|
+      {h1:success1:                                         }|
+      {h1:                ✅}                                |
                                                         |
-      {31:report 1}                                          |
-      - {32:OK} everything is fine                           |
+      {h2:report 1}                                          |
+      - ✅ {32:OK} everything is fine                        |
                                                         |
-      {31:report 2}                                          |
-      - {32:OK} nothing to see here                          |
+      {h2:report 2}                                          |
     ## grid 3
                                                         |
     ]],
@@ -250,14 +340,14 @@ describe(':checkhealth window', function()
   end)
 
   local function test_health_vsplit(left, emptybuf, mods)
-    local screen = Screen.new(50, 20)
+    local screen = Screen.new(50, 20, { ext_multigrid = true })
     screen:set_default_attr_ids {
+      h1 = { reverse = true },
+      h2 = { foreground = tonumber('0x6a0dad') },
       [1] = { foreground = Screen.colors.Blue, bold = true },
       [14] = { foreground = Screen.colors.LightGrey, background = Screen.colors.DarkGray },
-      [31] = { foreground = tonumber('0x6a0dad') },
       [32] = { foreground = Screen.colors.PaleGreen2 },
     }
-    screen:attach({ ext_multigrid = true })
     if not emptybuf then
       insert('hello')
     end
@@ -274,19 +364,22 @@ describe(':checkhealth window', function()
                                                         |
     ## grid 4
       ^                         |
-      {14:─────────────────────────}|*3
-      {14:───}                      |
-      {31:test_plug.success1:      }|
-      {31:require("test_plug.      }|
-      {31:success1.health").check()}|
+      {14:                         }|*3
+      {14:   }                      |
+      {h1:test_plug.               }|
+      {h1:success1:                }|
+      {h1:                         }|
+      {h1:                ✅}       |
                                |
-      {31:report 1}                 |
-      - {32:OK} everything is fine  |
+      {h2:report 1}                 |
+      - ✅ {32:OK} everything is    |
+      fine                     |
                                |
-      {31:report 2}                 |
-      - {32:OK} nothing to see here |
+      {h2:report 2}                 |
+      - ✅ {32:OK} nothing to see   |
+      here                     |
                                |
-      {1:~                        }|*4
+      {1:~                        }|
     ]]):format(
         left and '[4:-------------------------]│[2:------------------------]|*19'
           or '[2:------------------------]│[4:-------------------------]|*19',
@@ -314,8 +407,7 @@ describe(':checkhealth window', function()
   end
 
   local function test_health_split(top, emptybuf, mods)
-    local screen = Screen.new(50, 25)
-    screen:attach({ ext_multigrid = true })
+    local screen = Screen.new(50, 25, { ext_multigrid = true })
     screen._default_attr_ids = nil
     if not emptybuf then
       insert('hello')
@@ -333,17 +425,17 @@ describe(':checkhealth window', function()
                                                         |
     ## grid 4
       ^                                                  |
-      ──────────────────────────────────────────────────|
-      ────────────────────────────                      |
-      test_plug.success1: require("test_plug.success1.  |
-      health").check()                                  |
+                                                        |
+                                                        |
+      test_plug.                                        |
+      success1:                                         |
+                      ✅                                |
                                                         |
       report 1                                          |
-      - OK everything is fine                           |
+      - ✅ OK everything is fine                        |
                                                         |
       report 2                                          |
-      - OK nothing to see here                          |
-                                                        |
+      - ✅ OK nothing to see here                       |
     ]]):format(
         top
             and [[
@@ -385,7 +477,7 @@ describe(':checkhealth window', function()
     command('file my_buff')
     command('checkhealth success1')
     -- define a function that collects all buffers in each tab
-    -- returns a dictionary like {tab1 = ["buf1", "buf2"], tab2 = ["buf3"]}
+    -- returns a dict like {tab1 = ["buf1", "buf2"], tab2 = ["buf3"]}
     source([[
         function CollectBuffersPerTab()
                 let buffs = {}
