@@ -1,7 +1,7 @@
 local t = require('test.testutil')
 local n = require('test.functional.testnvim')()
 local Screen = require('test.functional.ui.screen')
-local tt = require('test.functional.terminal.testutil')
+local tt = require('test.functional.testterm')
 
 local clear, insert = n.clear, n.insert
 local command = n.command
@@ -17,12 +17,7 @@ describe('ext_hlstate detailed highlights', function()
     clear()
     command('syntax on')
     command('hi VertSplit gui=reverse')
-    screen = Screen.new(40, 8)
-    screen:attach({ ext_hlstate = true })
-  end)
-
-  after_each(function()
-    screen:detach()
+    screen = Screen.new(40, 8, { ext_hlstate = true })
   end)
 
   it('work with combined UI and syntax highlights', function()
@@ -33,42 +28,42 @@ describe('ext_hlstate detailed highlights', function()
     api.nvim_buf_add_highlight(0, -1, 'Statement', 1, 5, -1)
     command('/th co')
 
-    screen:expect(
-      [[
+    screen:expect {
+      grid = [[
       these are {1:some} lines                    |
       ^wi{2:th }{4:co}{3:lorful text}                      |
       {5:~                                       }|*5
-      {8:search hit BOTTOM, continuing at TOP}{7:    }|
+      {8:search hit BOTTOM, continuing at TOP}{6:    }|
     ]],
-      {
+      attr_ids = {
         [1] = {
-          { foreground = Screen.colors.Magenta },
-          { { hi_name = 'Constant', kind = 'syntax' } },
+          { foreground = Screen.colors.Magenta1 },
+          { { kind = 'syntax', hi_name = 'Constant' } },
         },
         [2] = {
-          { background = Screen.colors.Yellow },
-          { { hi_name = 'Search', ui_name = 'Search', kind = 'ui' } },
+          { background = Screen.colors.Yellow1 },
+          { { kind = 'ui', ui_name = 'Search', hi_name = 'Search' } },
         },
         [3] = {
-          { bold = true, foreground = Screen.colors.Brown },
-          { { hi_name = 'Statement', kind = 'syntax' } },
+          { foreground = Screen.colors.Brown, bold = true },
+          { { kind = 'syntax', hi_name = 'Statement' } },
         },
         [4] = {
-          { bold = true, background = Screen.colors.Yellow, foreground = Screen.colors.Brown },
+          { background = Screen.colors.Yellow1, bold = true, foreground = Screen.colors.Brown },
           { 3, 2 },
         },
         [5] = {
-          { bold = true, foreground = Screen.colors.Blue1 },
-          { { hi_name = 'NonText', ui_name = 'EndOfBuffer', kind = 'ui' } },
+          { foreground = Screen.colors.Blue, bold = true },
+          { { kind = 'ui', ui_name = 'EndOfBuffer', hi_name = 'NonText' } },
         },
-        [6] = {
-          { foreground = Screen.colors.Red },
-          { { hi_name = 'WarningMsg', ui_name = 'WarningMsg', kind = 'ui' } },
+        [6] = { {}, { { kind = 'ui', ui_name = 'MsgArea', hi_name = 'MsgArea' } } },
+        [7] = {
+          { foreground = Screen.colors.Red1 },
+          { { kind = 'syntax', hi_name = 'WarningMsg' } },
         },
-        [7] = { {}, { { hi_name = 'MsgArea', ui_name = 'MsgArea', kind = 'ui' } } },
-        [8] = { { foreground = Screen.colors.Red }, { 7, 6 } },
-      }
-    )
+        [8] = { { foreground = Screen.colors.Red1 }, { 6, 7 } },
+      },
+    }
   end)
 
   it('work with cleared UI highlights', function()
@@ -229,10 +224,10 @@ describe('ext_hlstate detailed highlights', function()
       [6] = { { foreground = tonumber('0x40ffff'), fg_indexed = true }, { 5, 1 } },
       [7] = { {}, { { hi_name = 'MsgArea', ui_name = 'MsgArea', kind = 'ui' } } },
     })
-    command(("enew | call termopen(['%s'])"):format(testprg('tty-test')))
+    command(("enew | call jobstart(['%s'],{'term':v:true})"):format(testprg('tty-test')))
     screen:expect([[
       ^tty ready                               |
-      {1: }                                       |
+                                              |
                                               |*5
       {7:                                        }|
     ]])
@@ -247,7 +242,7 @@ describe('ext_hlstate detailed highlights', function()
       screen:expect([[
         ^tty ready                               |
         x {5:y z}                                   |
-        {1: }                                       |
+                                                |
                                                 |*4
         {7:                                        }|
       ]])
@@ -255,7 +250,7 @@ describe('ext_hlstate detailed highlights', function()
       screen:expect([[
         ^tty ready                               |
         x {2:y }{3:z}                                   |
-        {1: }                                       |
+                                                |
                                                 |*4
         {7:                                        }|
       ]])
@@ -273,7 +268,7 @@ describe('ext_hlstate detailed highlights', function()
     else
       screen:expect([[
         ^tty ready                               |
-        x {4:y}{2: }{3:z}                                   |
+        x {2:y }{3:z}                                   |
                                                 |*5
         {7:                                        }|
       ]])
@@ -507,9 +502,7 @@ describe('ext_hlstate detailed highlights', function()
     local num_lines = 500
     insert('first line\n')
     for _ = 1, num_lines do
-      insert([[
-        line
-      ]])
+      api.nvim_paste('  line\n', false, -1)
     end
     insert('last line')
 
